@@ -4,13 +4,6 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
-import numpy as np
-import os
-import pandas as pd
-import re
-import struct
-import thriftpy
-
 from .thrift_structures import parquet_thrift
 from .core import read_thrift
 from .writer import write
@@ -18,4 +11,4 @@ from . import core, schema, converted_types, api
 from .api import ParquetFile
 from .util import ParquetException
 
-__version__ = "0.1.0"
+__version__ = "0.1.2"
