@@ -5,7 +5,6 @@ from __future__ import print_function
 from __future__ import unicode_literals
 
 from collections import OrderedDict
-import os
 from six import text_type
 
 from .thrift_structures import parquet_thrift
@@ -50,6 +49,24 @@ def schema_to_text(root, indent=[]):
     return text
 
 
+def flatten(schema, root, name_parts=[]):
+    if not hasattr(schema, 'children'):
+        return
+    if schema is not root:
+        name_parts = name_parts + [schema.name]
+    # root.children.pop('.'.join(name_parts), None)
+    for name, item in schema.children.copy().items():
+        if schema.repetition_type == parquet_thrift.FieldRepetitionType.REPEATED:
+            continue
+        if len(getattr(item, 'children', [])) == 0:
+            root.children['.'.join(name_parts + [name])] = item
+        elif item.converted_type in [parquet_thrift.ConvertedType.LIST,
+                                     parquet_thrift.ConvertedType.MAP]:
+            root.children['.'.join(name_parts + [name])] = item
+        else:
+            flatten(item, root, name_parts)
+
+
 class SchemaHelper(object):
     """Utility providing convenience methods for schema_elements."""
 
@@ -60,9 +77,11 @@ class SchemaHelper(object):
         self.schema_elements_by_name = dict(
             [(se.name, se) for se in schema_elements])
         schema_tree(schema_elements)
+        self.text = schema_to_text(self.schema_elements[0])
+        flatten(self.root, self.root)
 
     def __str__(self):
-        return schema_to_text(self.schema_elements[0])
+        return self.text
 
     def __repr__(self):
         return "<Parquet Schema with {} entries>".format(
@@ -115,8 +134,10 @@ class SchemaHelper(object):
 
 
 def _is_list_like(helper, column):
+    if len(column.meta_data.path_in_schema) < 3:
+        return False
     se = helper.schema_element(
-        column.meta_data.path_in_schema[0:1])
+        column.meta_data.path_in_schema[:-2])
     ct = se.converted_type
     if ct != parquet_thrift.ConvertedType.LIST:
         return False
@@ -134,8 +155,10 @@ def _is_list_like(helper, column):
 
 
 def _is_map_like(helper, column):
+    if len(column.meta_data.path_in_schema) < 3:
+        return False
     se = helper.schema_element(
-        column.meta_data.path_in_schema[0:1])
+        column.meta_data.path_in_schema[:-2])
     ct = se.converted_type
     if ct != parquet_thrift.ConvertedType.MAP:
         return False
