@@ -176,7 +176,7 @@ def test_roundtrip_complex(tempdir, scheme,):
 def test_datetime_roundtrip(tempdir, df, capsys):
     fname = os.path.join(tempdir, 'test.parquet')
     w = False
-    if 'x' in df and str(df.x.dtype.tz) == 'Europe/London':
+    if 'x' in df and 'Europe/' in str(df.x.dtype.tz):
         with pytest.warns(UserWarning) as w:
             write(fname, df)
     else:
@@ -844,3 +844,19 @@ def test_cats_and_nulls(tempdir):
     out = pf.to_pandas()
     assert out.dtypes['x'] == 'category'
     assert out.x.tolist() == [1, 2, 1]
+
+
+def test_consolidate_cats(tempdir):
+    import json
+    df = pd.DataFrame({'x': pd.Categorical([1, 2, 1])})
+    fn = os.path.join(tempdir, 'temp.parq')
+    write(fn, df)
+    pf = ParquetFile(fn)
+    assert 2 == json.loads(pf.fmd.key_value_metadata[0].value)['columns'][0][
+        'metadata']['num_categories']
+    start = pf.row_groups[0].columns[0].meta_data.key_value_metadata[0].value
+    assert start == '2'
+    pf.row_groups[0].columns[0].meta_data.key_value_metadata[0].value = '5'
+    writer.consolidate_categories(pf.fmd)
+    assert 5 == json.loads(pf.fmd.key_value_metadata[0].value)['columns'][0][
+        'metadata']['num_categories']
