@@ -1,21 +1,20 @@
 from __future__ import print_function
 
-import io
 import json
 import numpy as np
-import os
 import pandas as pd
 import re
-import shutil
 import struct
-import sys
-import thriftpy
 import warnings
 
 import numba
 
-from thriftpy.protocol.compact import TCompactProtocol
-from thriftpy.protocol.exc import TProtocolException
+from .thrift_structures import parquet_thrift, write_thrift
+try:
+    from pandas.api.types import is_categorical_dtype
+except ImportError:
+    # Pandas <= 0.18.1
+    from pandas.core.common import is_categorical_dtype
 from .thrift_structures import parquet_thrift
 from .compression import compress_data, decompress_data
 from .converted_types import tobson
@@ -234,42 +233,6 @@ def time_shift(indata, outdata, factor=1000):  # pragma: no cover
             outdata[i] = indata[i] // factor
 
 
-def write_thrift(fobj, thrift):
-    """Write binary compact representation of thiftpy structured object
-
-    Parameters
-    ----------
-    fobj: open file-like object (binary mode)
-    thrift: thriftpy object to write
-
-    Returns
-    -------
-    Number of bytes written
-    """
-    t0 = fobj.tell()
-    pout = TCompactProtocol(fobj)
-    try:
-        thrift.write(pout)
-        fail = False
-    except TProtocolException as e:
-        typ, val, tb = sys.exc_info()
-        frames = []
-        while tb is not None:
-            frames.append(tb)
-            tb = tb.tb_next
-        frame = [tb for tb in frames if 'write_struct' in str(tb.tb_frame.f_code)]
-        variables = frame[0].tb_frame.f_locals
-        obj = variables['obj']
-        name = variables['fname']
-        fail = True
-    if fail:
-        raise ParquetException('Thrift parameter validation failure %s'
-                               ' when writing: %s-> Field: %s' % (
-            val.args[0], obj, name
-        ))
-    return fobj.tell() - t0
-
-
 def encode_plain(data, se):
     """PLAIN encoding; returns byte representation"""
     out = convert(data, se)
@@ -454,14 +417,14 @@ def write_column(f, data, selement, compression=None):
     encoding = "PLAIN"
 
     if has_nulls:
-        if str(data.dtype) == 'category':
+        if is_categorical_dtype(data.dtype):
             num_nulls = (data.cat.codes == -1).sum()
         elif data.dtype.kind in ['i', 'b']:
             num_nulls = 0
         else:
             num_nulls = len(data) - data.count()
         definition_data, data = make_definitions(data, num_nulls == 0)
-        if data.dtype.kind == "O" and str(data.dtype) != 'category':
+        if data.dtype.kind == "O" and not is_categorical_dtype(data.dtype):
             if selement.type == parquet_thrift.Type.INT64:
                 data = data.astype(int)
             elif selement.type == parquet_thrift.Type.BOOLEAN:
@@ -478,7 +441,7 @@ def write_column(f, data, selement, compression=None):
     diff = 0
     max, min = None, None
 
-    if str(data.dtype) == 'category':
+    if is_categorical_dtype(data.dtype):
         dph = parquet_thrift.DictionaryPageHeader(
                 num_values=len(data.cat.categories),
                 encoding=parquet_thrift.Encoding.PLAIN)
@@ -502,10 +465,16 @@ def write_column(f, data, selement, compression=None):
         try:
             if num_nulls == 0:
                 max, min = data.values.max(), data.values.min()
-                max = encode['PLAIN'](pd.Series([max]), selement)
-                min = encode['PLAIN'](pd.Series([min]), selement)
+                if selement.type == parquet_thrift.Type.BYTE_ARRAY:
+                    if selement.converted_type is not None:
+                        max = encode['PLAIN'](pd.Series([max]), selement)[4:]
+                        min = encode['PLAIN'](pd.Series([min]), selement)[4:]
+                else:
+                    max = encode['PLAIN'](pd.Series([max]), selement)
+                    min = encode['PLAIN'](pd.Series([min]), selement)
         except TypeError:
             pass
+        ncats = len(data.cat.categories)
         data = data.cat.codes
         cats = True
         encoding = "PLAIN_DICTIONARY"
@@ -519,8 +488,13 @@ def write_column(f, data, selement, compression=None):
     try:
         if encoding != 'PLAIN_DICTIONARY' and num_nulls == 0:
             max, min = data.values.max(), data.values.min()
-            max = encode['PLAIN'](pd.Series([max], dtype=data.dtype), selement)
-            min = encode['PLAIN'](pd.Series([min], dtype=data.dtype), selement)
+            if selement.type == parquet_thrift.Type.BYTE_ARRAY:
+                if selement.converted_type is not None:
+                    max = encode['PLAIN'](pd.Series([max]), selement)[4:]
+                    min = encode['PLAIN'](pd.Series([min]), selement)[4:]
+            else:
+                max = encode['PLAIN'](pd.Series([max]), selement)
+                min = encode['PLAIN'](pd.Series([min]), selement)
     except TypeError:
         pass
 
@@ -575,6 +549,10 @@ def write_column(f, data, selement, compression=None):
                 page_type=parquet_thrift.PageType.DICTIONARY_PAGE,
                 encoding=parquet_thrift.Encoding.PLAIN, count=1))
         cmd.dictionary_page_offset = dict_start
+        cmd.key_value_metadata.append(
+            parquet_thrift.KeyValue(key='num_categories', value=str(ncats)))
+        cmd.key_value_metadata.append(
+            parquet_thrift.KeyValue(key='numpy_dtype', value=str(data.dtype)))
     chunk = parquet_thrift.ColumnChunk(file_offset=offset,
                                        meta_data=cmd)
     write_thrift(f, chunk)
@@ -658,7 +636,7 @@ def make_metadata(data, has_nulls=True, ignore_columns=[], fixed_text=None,
         oencoding = (object_encoding if isinstance(object_encoding, STR_TYPE)
                      else object_encoding.get(column, None))
         fixed = None if fixed_text is None else fixed_text.get(column, None)
-        if str(data[column].dtype) == 'category':
+        if is_categorical_dtype(data[column].dtype):
             se, type = find_type(data[column].cat.categories,
                                  fixed_text=fixed, object_encoding=oencoding)
             se.name = column
@@ -674,7 +652,7 @@ def make_metadata(data, has_nulls=True, ignore_columns=[], fixed_text=None,
             se.repetition_type = parquet_thrift.FieldRepetitionType.OPTIONAL
         fmd.schema.append(se)
         root.num_children += 1
-    meta.value = json.dumps(pandas_metadata)
+    meta.value = json.dumps(pandas_metadata, sort_keys=True)
     return fmd
 
 
@@ -917,6 +895,7 @@ def write_common_metadata(fn, fmd, open_with=default_open,
         Strip out row groups from metadata before writing - used for "common
         metadata" files, containing only the schema.
     """
+    consolidate_categories(fmd)
     with open_with(fn, 'wb') as f:
         f.write(MARKER)
         if no_row_groups:
@@ -930,7 +909,26 @@ def write_common_metadata(fn, fmd, open_with=default_open,
         f.write(MARKER)
 
 
-def merge(file_list, verify_schema=True, open_with=default_open):
+def consolidate_categories(fmd):
+    key_value = [k for k in fmd.key_value_metadata
+                 if k.key == 'pandas'][0]
+    meta = json.loads(key_value.value)
+    cats = [c for c in meta['columns']
+            if 'num_categories' in (c['metadata'] or [])]
+    for cat in cats:
+        for rg in fmd.row_groups:
+            for col in rg.columns:
+                if ".".join(col.meta_data.path_in_schema) == cat['name']:
+                    ncats = [k.value for k in col.meta_data.key_value_metadata
+                             if k.key == 'num_categories']
+                    if ncats and int(ncats[0]) > cat['metadata'][
+                            'num_categories']:
+                        cat['metadata']['num_categories'] = int(ncats[0])
+    key_value.value = json.dumps(meta, sort_keys=True)
+
+
+def merge(file_list, verify_schema=True, open_with=default_open,
+          root=False):
     """
     Create a logical data-set out of multiple parquet files.
 
@@ -948,13 +946,18 @@ def merge(file_list, verify_schema=True, open_with=default_open):
     open_with: func
         Used for opening a file for writing as f(path, mode). If input list
         is ParquetFile instances, will be inferred from the first one of these.
+    root: str
+        If passing a list of files, the top directory of the data-set may
+        be ambiguous for partitioning where the upmost field has only one
+        value. Use this to specify the data'set root directory, if required.
 
     Returns
     -------
     ParquetFile instance corresponding to the merged data.
     """
     sep = sep_from_open(open_with)
-    basepath, fmd = metadata_from_many(file_list, verify_schema, open_with)
+    basepath, fmd = metadata_from_many(file_list, verify_schema, open_with,
+                                       root=root)
 
     out_file = sep.join([basepath, '_metadata'])
     write_common_metadata(out_file, fmd, open_with, no_row_groups=False)
